package aexp.stabilizedcompass;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class ArrowPanel extends SurfaceView implements SurfaceHolder.Callback {
	static final double ARROWLENGTH = 15.0;
	static final double ARROWDEG = 30.0 / 180.0 * Math.PI;
	
	public ArrowPanel(Context context, AttributeSet attrs) {
		super(context, attrs);
        getHolder().addCallback(this);
        setFocusable(true);
	}

// SurfaceHolder.Callback
	@Override
	public void surfaceChanged(
			SurfaceHolder holder, 
			int format, 
			int width,
			int height) {
		this.panelHeight = (float)height;
		this.panelWidth = (float)width;
		Canvas c = holder.lockCanvas();
		drawArrows( c );
		holder.unlockCanvasAndPost( c );
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		StabilizedCompassActivity.arrowPanel = this;
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		StabilizedCompassActivity.arrowPanel = this;		
	}

	public void drawWhiteArrow( Canvas c, double degree ) {
		whiteSet = true;
		whiteDegree = degree;
		recalculateWhite();
		drawArrows( c );
	}

	public void drawRedArrow( Canvas c, double degree ) {
		redSet = true;
		redDegree = degree;
		recalculateRed();
		drawArrows( c );
	}
	
	public void drawArrows( Canvas c ) {
		c.drawRGB( 0,0,0);
		Paint p = new Paint();
		p.setColor( Color.WHITE);
		float halfWidth = panelWidth / 2.0f;
		float halfHeight = panelHeight / 2.0f;
		c.drawLine( halfWidth,0,halfWidth,panelHeight,p);
		c.drawLine( 0,halfHeight,panelWidth,halfHeight,p);
		if( redSet ) {
			Paint redPaint = new Paint();
			redPaint.setColor( Color.RED );
			drawArrow( c,redPaint,halfWidth,halfHeight, redXOffset, redYOffset, redDegree );
		}
		if( whiteSet ) {
			Paint whitePaint = new Paint();
			whitePaint.setColor( Color.WHITE );
			drawArrow( c,whitePaint,halfWidth,halfHeight, whiteXOffset, whiteYOffset, whiteDegree );
		}
	}

	private void recalculateWhite() {
		float halfWidth = panelWidth / 2.0f;
		float halfHeight = panelHeight / 2.0f;
		float radius = halfWidth < halfHeight ?
							halfWidth :
							halfHeight;
		whiteXOffset = ((float)Math.cos( whiteDegree ))*radius; 
		whiteYOffset = ((float)Math.sin( whiteDegree ))*radius; 
	}

	private void recalculateRed() {
		float halfWidth = panelWidth / 2.0f;
		float halfHeight = panelHeight / 2.0f;
		float radius = halfWidth < halfHeight ?
							halfWidth :
							halfHeight;
		redXOffset = ((float)Math.cos( redDegree ))*radius; 
		redYOffset = ((float)Math.sin( redDegree ))*radius; 
	}

	private void drawArrow( 
			Canvas c, 
			Paint p, 
			float startx, 
			float starty, 
			float offsetx, 
			float offsety, 
			double degree ) {
		float endx = startx+offsetx;
		float endy = starty-offsety;
		c.drawLine( startx,starty,endx,endy,p);
		double adeg = Math.PI + degree - ARROWDEG;
		double aXOffs = Math.cos( adeg )*ARROWLENGTH;
		double aYOffs = Math.sin( adeg )*ARROWLENGTH;
		c.drawLine( endx,
					endy,
					(float)( endx+aXOffs ),
					(float)( endy-aYOffs ),
					p);
		adeg = Math.PI + degree + ARROWDEG;
		aXOffs = Math.cos( adeg )*ARROWLENGTH;
		aYOffs = Math.sin( adeg )*ARROWLENGTH;
		c.drawLine( endx,
					endy,
					(float)( endx+aXOffs ),
					(float)( endy-aYOffs ),
					p);
	}
	
	float panelHeight;
	float panelWidth;
	boolean redSet = false;
	double redDegree;
	float redXOffset,redYOffset;	
	boolean whiteSet = false;
	float whiteXOffset,whiteYOffset;
	double whiteDegree;
}
