package aexp.stabilizedcompass;

import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.ComponentName;
import android.content.Context;
import android.graphics.Canvas;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.DeadObjectException;
import android.os.Handler;
import android.os.RemoteException;
import android.os.IBinder;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.View;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.TextView;
import android.widget.SeekBar;
import android.widget.Button;

public class StabilizedCompassActivity extends Activity 
{
	static final String LOG_TAG = "STABILIZEDCOMPASS";
	static final String SAMPLING_SERVICE_ACTIVATED_KEY = "samplingServiceActivated";
	static final String STABILIZEDCOMPASS_KEY = "sampleCounter";

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        uiHandler = new Handler();
        setContentView(R.layout.main);
		Log.d( LOG_TAG, "onCreate" );
		if( savedInstanceState != null ) {
			sampleCounterText = savedInstanceState.getString( STABILIZEDCOMPASS_KEY );
			samplingServiceActivated = savedInstanceState.getBoolean( SAMPLING_SERVICE_ACTIVATED_KEY, false );
        } else
            samplingServiceActivated = false;
        Log.d( LOG_TAG, "onCreate; samplingServiceActivated: "+samplingServiceActivated );
		bindSamplingService();
        setContentView( R.layout.main );
		SensorManager sensorManager = 
				(SensorManager)getSystemService( SENSOR_SERVICE  );
		updateSensorName( R.id.accelsensorname,
				sensorManager,
				Sensor.TYPE_ACCELEROMETER,
				"Accelerometer");
		updateSensorName( R.id.compasssensorname,
				sensorManager,
				Sensor.TYPE_MAGNETIC_FIELD,
				"Gyroscope");
		sampleCounterTV = (TextView)findViewById( R.id.samplecounter );
		if( sampleCounterText != null ) {
			sampleCounterTV.setText( sampleCounterText );
			sampleCounterTV.setVisibility( View.VISIBLE );
        }
        arrowPanel = (ArrowPanel)findViewById( R.id.arrow);
		statusMessageTV = (TextView)findViewById( R.id.state );
		rotateXTV = (TextView)findViewById( R.id.rotate_x);
		rotateYTV = (TextView)findViewById( R.id.rotate_y);
		rotateZTV = (TextView)findViewById( R.id.rotate_z);
		CheckBox cb = (CheckBox)findViewById( R.id.cb_sampling );
        if( samplingServiceActivated )
            cb.setChecked( true );
        else
            stopSamplingService();
		cb.setOnClickListener( new View.OnClickListener() {			
			@Override
			public void onClick(View v) {
				CheckBox cb = (CheckBox)v;
				boolean isChecked = cb.isChecked();
				if( isChecked ) {
					Log.d( LOG_TAG, "sampling activated" );
					samplingServiceActivated = true;
					startSamplingService();
			        sampleCounterTV.setText( sampleCounterText );
					sampleCounterTV.setVisibility( View.VISIBLE );
				} else {
					Log.d( LOG_TAG, "sampling deactivated" );
					samplingServiceActivated = false;
					sampleCounterTV.setVisibility( View.INVISIBLE );
					stopSamplingService();
				}
				
			}
		});
    }
    
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState( outState );
		Log.d( LOG_TAG, "onSaveInstanceState" );
		outState.putBoolean( SAMPLING_SERVICE_ACTIVATED_KEY, samplingServiceActivated ); 
		if( sampleCounterText != null )
			outState.putString( STABILIZEDCOMPASS_KEY, sampleCounterText );
	}

	protected void onDestroy() {
		super.onDestroy();
		Log.d( LOG_TAG, "onDestroy" );
		releaseSamplingService();
	}

	private void updateSensorName(
			int tvID,
			SensorManager sensorManager,
			int sensorType,
			String sensorName) {
		TextView tv = (TextView)findViewById( tvID );
		List<Sensor> sensors = 
				sensorManager.
					getSensorList( 
						sensorType);
		StringBuffer sb = new StringBuffer();
		if( sensors.size() == 0 ) {
			sb.append( sensorName );
			sb.append( ": N/A");
		} else {
			Sensor s = sensors.get(0);
			sb.append( s.getName());
		}
		tv.setText( new String( sb ));
	}
	
	private void startSamplingService() {
		if( samplingServiceRunning )	// shouldn't happen
			stopSamplingService();
        sampleCounterText = "0";
        Intent i = new Intent();
        i.setClassName( "aexp.stabilizedcompass","aexp.stabilizedcompass.SamplingService" );
        startService( i );
		samplingServiceRunning = true;				
	}

	private void stopSamplingService() {
		Log.d( LOG_TAG, "stopSamplingService" );
		if( samplingServiceRunning ) {
			stopSampling();
			samplingServiceRunning = false;
		}
	}

	private void bindSamplingService() {
		samplingServiceConnection = new SamplingServiceConnection();
		Intent i = new Intent();
		i.setClassName( "aexp.stabilizedcompass", "aexp.stabilizedcompass.SamplingService" );
		bindService( i, samplingServiceConnection, Context.BIND_AUTO_CREATE);
	}

	private void releaseSamplingService() {
		releaseCallbackOnService();
		unbindService( samplingServiceConnection );	  
		samplingServiceConnection = null;
	}


    private void setCallbackOnService() {
		if( samplingService == null )
			Log.e( LOG_TAG, "setCallbackOnService: Service not available" );
		else {
			try {
				samplingService.setCallback( iStabilizedCompass.asBinder() );
			} catch( DeadObjectException ex ) {
				Log.e( LOG_TAG, "DeadObjectException",ex );
			} catch( RemoteException ex ) {
				Log.e( LOG_TAG, "RemoteException",ex );
			}
		}
	}

    private void releaseCallbackOnService() {
		if( samplingService == null )
			Log.e( LOG_TAG, "releaseCallbackOnService: Service not available" );
		else {
			try {
				samplingService.removeCallback();
			} catch( DeadObjectException ex ) {
				Log.e( LOG_TAG, "DeadObjectException",ex );
			} catch( RemoteException ex ) {
				Log.e( LOG_TAG, "RemoteException",ex );
			}
		}
	}

    private void updateSamplingServiceRunning() {
		if( samplingService == null )
			Log.e( LOG_TAG, "updateSamplingServiceRunning: Service not available" );
		else {
			try {
				samplingServiceRunning = samplingService.isSampling();
			} catch( DeadObjectException ex ) {
				Log.e( LOG_TAG, "DeadObjectException",ex );
			} catch( RemoteException ex ) {
				Log.e( LOG_TAG, "RemoteException",ex );
			}
		}
	}

    private void updateState() {
		if( samplingService == null )
			Log.e( LOG_TAG, "updateState: Service not available" );
		else {
			try {
				state = samplingService.getState();
			} catch( DeadObjectException ex ) {
				Log.e( LOG_TAG, "DeadObjectException",ex );
			} catch( RemoteException ex ) {
				Log.e( LOG_TAG, "RemoteException",ex );
			}
		}
	}
        
    private void stopSampling() {
		Log.d( LOG_TAG, "stopSampling" );
		if( samplingService == null )
			Log.e( LOG_TAG, "stopSampling: Service not available" );
		else {
			try {
				samplingService.stopSampling();
			} catch( DeadObjectException ex ) {
				Log.e( LOG_TAG, "DeadObjectException",ex );
			} catch( RemoteException ex ) {
				Log.e( LOG_TAG, "RemoteException",ex );
			}
		}
	}

    private String getStateName( int state ) {
		String stateName = null;
		switch( state ) {
		case SamplingService.ENGINESTATES_IDLE:
			stateName = "Idle";
			break;

		case SamplingService.ENGINESTATES_CALIBRATING:
			stateName = "Calibrating";
			break;
			
		case SamplingService.ENGINESTATES_MEASURING:
			stateName = "Measuring";
			break;
			
		default:
			stateName = "N/A";
			break;
		}
		return stateName;
	}

    private IStabilizedCompass.Stub iStabilizedCompass 
				= new IStabilizedCompass.Stub() {

		@Override
		public void sampleCounter(int count) throws RemoteException {
			Log.d( LOG_TAG, "sample count: "+count );
			sampleCounterText = Integer.toString( count );
			sampleCounterTV.setText( sampleCounterText );			
		}
		
		public void statusMessage( int newState ) {
			Log.d(LOG_TAG, "statusMessage: "+newState );
			if( newState < SamplingService.ENGINESTATES_TRANSIENT ) {
				state = newState;
				if( statusMessageTV  != null ) {
					statusMessageTV.setText( getStateName( newState ) );
				}
			}
			switch( newState ) {
			case SamplingService.ENGINESTATES_IDLE:
				rotateXTV.setVisibility( View.GONE);
				rotateYTV.setVisibility( View.GONE);
				rotateZTV.setVisibility( View.GONE);
				arrowPanel.setVisibility( View.GONE);
				break;
				
			case SamplingService.ENGINESTATES_CALIBRATING:
				rotateXTV.setVisibility( View.VISIBLE);
				rotateYTV.setVisibility( View.VISIBLE);
				rotateZTV.setVisibility( View.VISIBLE);
				arrowPanel.setVisibility( View.GONE);
				break;
			
			case SamplingService.ENGINESTATES_MEASURING:
				rotateXTV.setVisibility( View.GONE);
				rotateYTV.setVisibility( View.GONE);
				rotateZTV.setVisibility( View.GONE);
				arrowPanel.setVisibility( View.VISIBLE);
				break;
				
			case SamplingService.ENGINESTATES_XGONE:
				rotateXTV.setVisibility( View.INVISIBLE);
				break;

			case SamplingService.ENGINESTATES_YGONE:
				rotateYTV.setVisibility( View.INVISIBLE);
				break;

			case SamplingService.ENGINESTATES_ZGONE:
				rotateZTV.setVisibility( View.INVISIBLE);
				break;
				
			}
		}

		@Override
		public void whiteArrow( double degree ) throws RemoteException {
			if( arrowPanel != null ) {
				SurfaceHolder holder = arrowPanel.getHolder();
				Canvas c = holder.lockCanvas();
				if( c != null ) {
					arrowPanel.drawWhiteArrow(c, degree);
					holder.unlockCanvasAndPost(c);
				}
			}
		}

		@Override
		public void redArrow( double degree ) throws RemoteException {
			if( arrowPanel != null ) {
				SurfaceHolder holder = arrowPanel.getHolder();
				Canvas c = holder.lockCanvas();
				if( c != null ) {
					arrowPanel.drawRedArrow(c, degree);
					holder.unlockCanvasAndPost(c);
				}
			}
		}
    
    };

    private Handler uiHandler;
    private int state = SamplingService.ENGINESTATES_IDLE;
    private ISamplingService samplingService = null;
    private SamplingServiceConnection samplingServiceConnection = null;
	private boolean samplingServiceRunning = false;
    private boolean samplingServiceActivated = false;
	private TextView sampleCounterTV;
	private TextView statusMessageTV;
	private String sampleCounterText = null;
	public static ArrowPanel arrowPanel = null;
	private TextView rotateXTV;
	private TextView rotateYTV;
	private TextView rotateZTV;
	
    class SamplingServiceConnection implements ServiceConnection {
        public void onServiceConnected(ComponentName className, 
			IBinder boundService ) {
			Log.d( LOG_TAG, "onServiceConnected" );        	
        	samplingService = ISamplingService.Stub.asInterface((IBinder)boundService);
	    	setCallbackOnService();
			updateSamplingServiceRunning();
			updateState();
/*
            if( !samplingServiceRunning )
                startSamplingService();
*/                
		    CheckBox cb = (CheckBox)findViewById( R.id.cb_sampling );
            cb.setChecked( samplingServiceRunning );
			if( statusMessageTV  != null ) {
				statusMessageTV.setText( getStateName( state ) );
			}
			if( state == SamplingService.ENGINESTATES_MEASURING )
				arrowPanel.setVisibility( View.VISIBLE);
		 	Log.d( LOG_TAG,"onServiceConnected" );
        }

        public void onServiceDisconnected(ComponentName className) {
        	samplingService = null;
			Log.d( LOG_TAG,"onServiceDisconnected" );
        }
    };


}
